@@ -3,7 +3,7 @@
 import logging
 from logging import Logger, StreamHandler
 
-from . import api, errors, geography, query, socrata, utilities  # noqa
+from . import api, constants, errors, geography, query, socrata, utilities  # noqa
 from .query import Query  # noqa
 from .utilities import clear_cache  # noqa
 
